unit Evnt_pgm;

interface

uses
  DisplayUnit,
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, Menus,
  wsc, ExtCtrls, StdCtrls;
type
  TEvnt = class(TForm)
    MainMenu: TMainMenu;
    menuChange: TMenuItem;
    menuPort: TMenuItem;
    menuBaud: TMenuItem;
    menuCOM1: TMenuItem;
    menuCOM2: TMenuItem;
    menuCOM3: TMenuItem;
    menuCOM4: TMenuItem;
    menu2400: TMenuItem;
    menu9600: TMenuItem;
    menu19200: TMenuItem;
    menu38400: TMenuItem;
    menu57600: TMenuItem;
    MemoMain: TMemo;
    menu115200: TMenuItem;
    menuOnLine: TMenuItem;
    menuExit: TMenuItem;
    procedure FormCreate(Sender: TObject);
    procedure menuOnLineClick(Sender: TObject);
    procedure menuCOM1Click(Sender: TObject);
    procedure menuCOM2Click(Sender: TObject);
    procedure menuCOM3Click(Sender: TObject);
    procedure menuCOM4Click(Sender: TObject);
    procedure menuExitClick(Sender: TObject);
    procedure menu2400Click(Sender: TObject);
    procedure menu9600Click(Sender: TObject);
    procedure menu19200Click(Sender: TObject);
    procedure menu38400Click(Sender: TObject);
    procedure menu57600Click(Sender: TObject);
    procedure menu115200Click(Sender: TObject);
    procedure MyKeyPress(Sender: TObject; var Key: Char);

  private
    { Private declarations }

  public
    { Public declarations }

  end ;

var
  {globals}
  Evnt: TEvnt;
  Port : Integer;
  Baud : Integer;
  Parity : Integer;
  DataBits : Integer;
  StopBits : Integer;
  ThreadID : DWORD;
  RxBuffer : PChar;

implementation

{$R *.DFM}

procedure GetIncoming;
var
  S : String;
  CharCount : Integer;
begin
  {Gather all incoming serial data}
  CharCount := SioGets(Port,RxBuffer,1024);
  if CharCount > 0 then
  begin
    {convert to string}
    RxBuffer[CharCount] := Chr(0);
    S := StrPas(RxBuffer);
    {display}
    DisplayString(Evnt.MemoMain,S);
  end
end; {GetIncoming}

function DoThread(Dummy:Pointer):Integer;
begin
  while true do  begin
    {block until data is ready}
    SioEvent(Port, EV_RXCHAR);
    {read all available data}
    GetIncoming;
  end
end; {DoThread}

procedure TEvnt.FormCreate(Sender: TObject);
begin
 (* initialize parameters *)
  Port := COM1;
  Baud := Baud19200;
  Parity := NoParity;
  DataBits := WordLength8;
  StopBits := OneStopBit;
  ThreadID := 0;
  GetMem(RxBuffer, 1024);
  (* initialize menu settings *)
  menuCOM1.Checked := true;
  menu19200.Checked := true
end;

procedure TEvnt.menuOnLineClick(Sender: TObject);
var
  Code : Integer;
begin
  (* initialize WSC32 *)
  Code := SioReset(Port,2048,1024);
  if Code < 0 then
    begin
      DisplayString(Evnt.MemoMain,Format('Error %d: Cannot reset port',[Code]));
      exit
    end;
  (* update menu settings *)
  Evnt.Caption := 'EVENT: COM' + Chr($31+Port) + ' Online';
  menuOnLine.Enabled := false;
  menuChange.Enabled := false;
  SioBaud(Port, Baud);
  SioParms(Port, Parity, StopBits, DataBits);
  SioDTR(Port,'S');
  SioRTS(Port,'S');
  (* create thread to get incoming serial data *)
  CreateThread(nil,0,@DoThread,nil,0,ThreadID);
  MemoMain.SetFocus
end;

procedure TEvnt.menuCOM1Click(Sender: TObject);
begin
  menuCOM1.Checked := true;
  menuCOM2.Checked := false;
  menuCOM3.Checked := false;
  menuCOM4.Checked := false;
  Port := COM1
end;

procedure TEvnt.menuCOM2Click(Sender: TObject);
begin
  menuCOM1.Checked := false;
  menuCOM2.Checked := true;
  menuCOM3.Checked := false;
  menuCOM4.Checked := false;
  Port := COM2
end;

procedure TEvnt.menuCOM3Click(Sender: TObject);
begin
  menuCOM1.Checked := false;
  menuCOM2.Checked := false;
  menuCOM3.Checked := true;
  menuCOM4.Checked := false;
  Port := COM3
end;

procedure TEvnt.menuCOM4Click(Sender: TObject);
begin
  menuCOM1.Checked := false;
  menuCOM2.Checked := false;
  menuCOM3.Checked := false;
  menuCOM4.Checked := true;
  Port := COM4
end;

procedure TEvnt.menuExitClick(Sender: TObject);
begin
  SioDone(Port);
  Freemem(RxBuffer,1024);
  Application.Terminate;
end;

procedure TEvnt.menu2400Click(Sender: TObject);
begin
  menu2400.Checked := true;
  menu9600.Checked := false;
  menu19200.Checked := false;
  menu38400.Checked := false;
  menu57600.Checked := false;
  menu115200.Checked := false;
  Baud := Baud2400
end;

procedure TEvnt.menu9600Click(Sender: TObject);
begin
  menu2400.Checked := false;
  menu9600.Checked := true;
  menu19200.Checked := false;
  menu38400.Checked := false;
  menu57600.Checked := false;
  menu115200.Checked := false;
  Baud := Baud9600
end;

procedure TEvnt.menu19200Click(Sender: TObject);
begin
  menu2400.Checked := false;
  menu9600.Checked := false;
  menu19200.Checked := true;
  menu38400.Checked := false;
  menu57600.Checked := false;
  menu115200.Checked := false;
  Baud := Baud19200
end;

procedure TEvnt.menu38400Click(Sender: TObject);
begin
  menu2400.Checked := false;
  menu9600.Checked := false;
  menu19200.Checked := false;
  menu38400.Checked := true;
  menu57600.Checked := false;
  menu115200.Checked := false;
  Baud := Baud38400
end;

procedure TEvnt.menu57600Click(Sender: TObject);
begin
  menu2400.Checked := false;
  menu9600.Checked := false;
  menu19200.Checked := false;
  menu38400.Checked := false;
  menu57600.Checked := true;
  menu115200.Checked := false;
  Baud := Baud57600
end;

procedure TEvnt.menu115200Click(Sender: TObject);
begin
  menu2400.Checked := false;
  menu9600.Checked := false;
  menu19200.Checked := false;
  menu38400.Checked := false;
  menu57600.Checked := false;
  menu115200.Checked := true;
  Baud := Baud115200
end;

procedure TEvnt.MyKeyPress(Sender: TObject; var Key: Char);
begin
  SioPutc(Port,Key)
end;

end.
